from os import listdir, remove, rename, kill
from signal import SIGTERM
from os.path import isfile, exists, getsize, isdir
from datetime import datetime
from shutil import rmtree
from subprocess import Popen
import pickle
from time import sleep

from gwcommands import TAR
from variables import BASE_DIR, DATA_DIR, QUEUED_DIR, WAIT_FILE, \
    STAY_AWAKE_FILE, SPECIAL_FILE, BOOT_FILE, INFO_FILE, MIN_AWAKE_TIME,\
    GEOPHONE_SPECIAL_FILE, PROBE_SPECIAL_FILE, MAX_SPECIAL_TIME
from gwlogging import logger, gw_stdout, gw_stderr
from GwErrors import GwConfigError
from gwtimeutils import totimestamp


def writeFile(filename, contents):
    try:
        f = open(filename, "w")
        for l in contents:
            f.write("%s\n" % l)
        f.close()
    except IOError as e:
        logger.error("Error occured on writing %s" % filename)
        print type(e)
        print e.args


def packFiles(mode):
    """pack the files into a tarball ready for sending"""
    now = datetime.utcnow()

    # Rename data files to be unique, prepend date.
    logger.debug("Renaming data files to be unique")
    for fname in listdir(BASE_DIR + DATA_DIR):
        if isfile("%s/%s" % (BASE_DIR + DATA_DIR, fname)):
            rename("%s/%s" % (BASE_DIR + DATA_DIR, fname),
                        "%s/%i-%2.2i-%2.2i-%2.2i.%2.2i-%s" % (
                        BASE_DIR + DATA_DIR,
                        now.year, now.month, now.day, now.hour,
                        now.minute, fname))

    # Create the new tar filename:
    tarf = "%s_data_%i-%2.2i-%2.2i-%2.2i.%2.2i.tar.gz" % (mode, now.year,
                now.month, now.day, now.hour, now.minute)

    # Tar up stuff
    p = Popen(args="%s -czf ./%s/%s ./%s" % (TAR, QUEUED_DIR,
                            tarf, DATA_DIR),
                            cwd=BASE_DIR, shell=True, stdout=gw_stdout,
                            stderr=gw_stderr)
    p.wait()

    # Delete the tarred files
    for fname in listdir(BASE_DIR + DATA_DIR):
        if isfile("%s/%s" % (BASE_DIR + DATA_DIR, fname)):
            remove("%s/%s" % (BASE_DIR + DATA_DIR, fname))
        elif isdir("%s/%s" % (BASE_DIR + DATA_DIR, fname)):
            rmtree("%s/%s" % (BASE_DIR + DATA_DIR, fname), True)
    logger.info("Tar created and tarred files deleted.")


def wait():
    return exists(WAIT_FILE)


def stay_awake(start_time):
    """Returns True if we need to stay awake."""
    # Always stay awake if this file exists
    if exists(STAY_AWAKE_FILE):
        return True
    # Always stay awake for some period of time
    if (datetime.utcnow() - start_time) < MIN_AWAKE_TIME:
        return True
    else:
        return False


# Execute special file
def executeSpecial(fname=(BASE_DIR + DATA_DIR +"/" +SPECIAL_FILE)):
    logger.debug(fname)
    if exists(fname):
        start = datetime.utcnow()
        finish = start + MAX_SPECIAL_TIME
        logger.info("Executing special file:%s" % fname)
        special = Popen(args="bash %s" % fname,
                    cwd=(BASE_DIR + DATA_DIR),
                    shell=True,
                    stdout=gw_stdout,
                    stderr=gw_stderr)
        while special.poll() is None:
            if datetime.utcnow() > finish:
                logger.error("Forcing termination of special")
                kill(special.pid, SIGTERM)
                special.wait()
            else:
                sleep(1)
        special.wait()
        logger.info("Time taken for special %s" % 
            (str(datetime.utcnow() - start)))

def executeProbeSpecial():
    executeSpecial(BASE_DIR + DATA_DIR + "/" + PROBE_SPECIAL_FILE)

def executeGeophoneSpecial():
    executeSpecial(BASE_DIR + DATA_DIR + "/" + GEOPHONE_SPECIAL_FILE)


def file_size(filename):
    """Returns the file size of the specified file"""
    return getsize(filename)


def increment_boot_count():
    try:
        f = open(BOOT_FILE, "r")
        for l in f:
            l = l.strip()
            if len(l) > 0:
                boot_count = int(l)
                break
        f.close()
    except IOError:
        logger.error("Boot file not read - assuming first boot")
        boot_count = 0  # will be incremented later
    if boot_count > 1600:
        logger.error("Boot count:%d" % boot_count)
        logger.error("Replace soon")
    elif boot_count > 1000:
        logger.warn("Boot count:%d" % boot_count)
    else:
        logger.info("Boot count %d" % boot_count)
    boot_count += 1
    f = open(BOOT_FILE, "w")
    f.write(str(boot_count) + "\n")
    f.close()


def get_last_run_time():
    """Reads the last time of execution from a file
    Returns a datetime object"""
    try:
        f = open(INFO_FILE, "r")
        t = None
        for l in f:
            l = l.strip()
            if len(l) > 0:
                t = datetime.utcfromtimestamp(int(l))
                break
        f.close()
        return t
    except IOError:
        logger.error("Info file not read")
        return None


def set_last_run_time():
    ifile = open(INFO_FILE, "w")
    ifile.write(str(totimestamp(datetime.utcnow())) + "\n")
    ifile.close()


def save_config(conf, filename):
    try:
        f = open(filename, "w")
        logger.debug("%s opened for writing" % filename)
        pickle.dump(conf, f)
        logger.info("Config saved to %s" % filename)
        f.close()
    except (pickle.PickleError, IOError):
        raise GwConfigError()

def load_config(filename):
    try:
        f = open(filename, "r")
        logger.debug("%s opened for reading" % filename)
        c = pickle.load(f)
        logger.info("Config loaded from %s" % filename)
        f.close()
    except (pickle.UnpicklingError, IOError):
        raise GwConfigError
    return c
